#ifndef X10I2C_H
#define X10I2C_H
#include <stdint.h>
#include "x10-i2c-debug.h"

typedef struct {
  int fd;  
  char filename[33];
  uint8_t current_bus;
  char s[128];
  dbg_desc_t dbg;
  uint8_t x10model;
  uint8_t sitara_bus;
  uint8_t sitara_saved_bus;
} x10i2c_t;

#define X10MODEL_BBONE   1
#define X10MODEL_SITARA  2
#define X10MODEL_SITARA2 3

#define X10_TCA9535PIN_TX_DIS  0x01
#define X10_TCA9535PIN_TX_FLT  0x02
#define X10_TCA9535PIN_RX_LOS  0x04
#define X10_TCA9535PIN_MOD_ABS 0x08
#define X10_TCA9535PIN_LINK_LED 0x10
#define X10_TCA9535PIN_ALM_LED 0x20
#define X10_TCA9535PIN_ACT_LED 0x40
#define X10_TCA9535PIN_GRND    0x80

/* Pin for model==3 */
#define X10_TCA9535PIN_RS0  0x10
#define X10_TCA9535PIN_RS1  0x20
#define X10_TCA9535PIN_PWOK 0x40

/* Fan constants */
#define X10FAN_CHIP 0
#define X10FAN_SYS  1
#define X10FAN_PWR0 2
#define X10FAN_PWR1 3

#define X10_PORTLED_ACT_NOBLINK  0x80

#define X10_SFPPIN_TX_FAULT    1
#define X10_SFPPIN_TX_DISABLE  2
#define X10_SFPPIN_RX_LOSS     4
#define X10_SFPPIN_RATE_SEL0   8
#define X10_SFPPIN_RATE_SEL1   0x10

int x10i2c_init(x10i2c_t **, uint8_t model, int dlevel);
int x10i2c_free(x10i2c_t *);

int x10i2c_init_hw(x10i2c_t *);
int x10i2c_set_port_tx(x10i2c_t *, uint8_t lp, uint8_t enable);
int x10i2c_get_port_tx(x10i2c_t *, uint8_t lp, uint8_t *enable);
int x10i2c_get_port_rxloss(x10i2c_t *, uint8_t lp, uint8_t *enable);

int x10i2c_set_port_linkled(x10i2c_t *, uint8_t lp, uint8_t enable);
int x10i2c_set_port_almled(x10i2c_t *, uint8_t lp, uint8_t enable);
int x10i2c_set_port_actled(x10i2c_t *, uint8_t lp, uint8_t enable);
int x10i2c_set_port_leds(x10i2c_t *, uint8_t lp, uint8_t leds);

int x10_test_ports_leds(x10i2c_t *, uint16_t timer);

int x10i2c_get_port_sfp_status(x10i2c_t *, uint8_t lp, uint8_t *enable);

int x10i2c_set_sfp_regs(x10i2c_t *h, uint8_t lp, uint8_t i2caddr, uint8_t reg, uint16_t count, void *data);
int x10i2c_get_sfp_regs(x10i2c_t *h, uint8_t lp, uint8_t i2caddr, uint8_t reg, uint16_t count, void *data);
int x10_get_sfp_link_status(x10i2c_t *h, uint16_t port, uint8_t *sts);

int x10_powerled_pin(x10i2c_t *h, uint8_t v);
int x10_reset_pin(x10i2c_t *h, uint8_t v);
int x10_reset(x10i2c_t *h);

int x10_set_port_rs0_out(x10i2c_t *h, uint8_t lp, uint8_t enable);
int x10_set_port_rs1_out(x10i2c_t *h, uint8_t lp, uint8_t enable);
int x10_get_port_rs0_out(x10i2c_t *h, uint8_t lp, uint8_t *enable);
int x10_get_port_rs1_out(x10i2c_t *h, uint8_t lp, uint8_t *enable);
int x10_get_port_sfp_pwrok(x10i2c_t *h, uint8_t lp, uint8_t *enable);

int x10_set_port_sfp_pins(x10i2c_t *h, uint8_t lp, uint16_t pins);
int x10_get_port_sfp_pins(x10i2c_t *h, uint8_t lp, uint16_t *pins);

/* Fujitsu register access */
int x10i2c_fujitsu_set_reg(x10i2c_t *h, uint16_t addr, uint32_t value);
int x10i2c_fujitsu_get_reg(x10i2c_t *h, uint16_t addr, uint32_t *value);
int x10i2c_fujitsu_set_port_reg(x10i2c_t *h, uint16_t pp, uint16_t addr, uint32_t value);
int x10i2c_fujitsu_get_port_reg(x10i2c_t *h, uint16_t pp, uint16_t addr, uint32_t *value);

int x10_set_display_light(x10i2c_t *h, uint8_t enable);
int x10_display_printf(x10i2c_t *h, char x, char y, char *s, ...);
int x10_display_clear_line(x10i2c_t *h, char y);
int x10_display_clear(x10i2c_t *h);

#define X10_TOBJID_CHIP 0
#define X10_TOBJID_SYS  1
#define X10_TOBJID_PWR0 2
#define X10_TOBJID_PWR1 3

int x10_set_fan_percent(x10i2c_t *h, uint8_t objid, uint8_t percent);
int x10_get_temp(x10i2c_t *h, uint8_t objid, uint8_t *temp);
int x10_is_present(x10i2c_t *h, uint8_t objid, uint8_t *present);

int x10i2c_print_all_sensors(x10i2c_t *h, FILE *fl);
int x10_read_eeprom(x10i2c_t *h, uint16_t memaddr, uint16_t count, void *data);
int x10_write_eeprom(x10i2c_t *h, uint16_t memaddr, uint16_t count, void *data);

typedef enum {
  X10_HW_TEST_DISPLAY,
  X10_HW_TEST_LEDS,
  X10_HW_TEST_LINK_LEDS,
  X10_HW_TEST_ACT_LEDS,
  X10_HW_TEST_ALM_LEDS,
  X10_HW_TEST_TEMPS,
  X10_HW_TEST_VOLTAGES,
  X10_HW_TEST_FANS,
  X10_HW_TEST_I2CBUS,
  X10_HW_TEST_I2CDEVICES,
  X10_HW_TEST_INIT,
  X10_HW_TEST_EEPROM,
  X10_HW_TEST_FUJITSU
} x10_hwtest_id_t;

int x10_hwtest(x10i2c_t *h, x10_hwtest_id_t id, FILE *fl);
int x10_hwtest_auto(x10i2c_t *h, FILE *fl, uint8_t stop);

typedef struct x10_eeprom_info_s {
  uint32_t serial;
  uint32_t reserved[3];
  uint32_t size;
  uint32_t sum;
  uint16_t year;
  uint8_t month;
  uint8_t  day;
  uint16_t ver;
  uint8_t reserved2;
  uint8_t hsum;
} x10_eeprom_info_t;

int x10_update_eeprom(x10i2c_t *h, FILE *out, char *fname, x10_eeprom_info_t *info);
int x10_check_eeprom(x10i2c_t *h, uint8_t *ok);
int x10_get_eeprom_info(x10i2c_t *h, x10_eeprom_info_t *info);


#endif

